@@ -1,5 +1,7 @@
+import ast
 import json
 import math
+import re
 from copy import deepcopy
 from functools import partial
 from types import FunctionType
@@ -8,7 +10,6 @@ from typing import Type, Tuple, Sequence, Dict, Callable, Any, Optional, List
 from django.apps import apps
 from django.conf.urls import url
 from django.db.models import QuerySet, Model
-from rest_framework import status
 from rest_framework.authentication import BaseAuthentication
 from rest_framework.parsers import FormParser, MultiPartParser
 from rest_framework.permissions import BasePermission
@@ -30,6 +31,26 @@ from .constructors import _construct_serializer, _construct_filter_backend
 from .namespace import _append_to_namespace, _RESOURCE_NAME_TO_SPICE, _MODEL_TO_SERIALIZERS
 from .types import CustomField, Filter, Relation, GenericRelation, ComputedFilter
 
+FILTER_REGEX = re.compile(r'filter\[(?P<field>[\w_\-]+)(?P<op>\.[\w_\-]+)?\]', re.IGNORECASE)
+FILTER_MAP = {
+    'is_null': 'is_null',
+    'is_not_null': 'is_not_null',
+    'eq': '==',
+    'ne': '!=',
+    'gt': '>',
+    'lt': '<',
+    'gte': '>=',
+    'gle': '<=',
+    'contains': 'like',
+    'icontains': 'ilike',
+    'not_icontains': 'not_ilike',
+    'not_contains': 'not_like',
+    'in': 'in',
+    'not_in': 'not_in',
+    'any': 'any',
+    'not_any': 'not_any',
+}
+
 
 class JsonApiModelViewBuilder:
     DEFAULT_RELATED_LIMIT = 100
@@ -230,7 +251,10 @@ class JsonApiModelViewBuilder:
 
     def _get_history_urls(self) -> Sequence[partial]:
         history_builder = deepcopy(self)
-        history_builder._skip_plugins = [plugins.DJANGO_SIMPLE_HISTORY, plugins.AUTO_ADMIN_VIEWS]
+        if plugins.AUTO_ADMIN_VIEWS not in history_builder._skip_plugins:
+            history_builder._skip_plugins = [plugins.DJANGO_SIMPLE_HISTORY]
+        else:
+            history_builder._skip_plugins = [plugins.DJANGO_SIMPLE_HISTORY, plugins.AUTO_ADMIN_VIEWS]
         history_builder._model = apps.get_model(self._model.objects.model._meta.db_table.split('_')[0],
                                                 f'Historical{self._model.__name__}')
         history_builder._queryset = self._model.history
@@ -453,9 +477,11 @@ class JsonApiResourceViewBuilder:
                  unique_identifier: Optional[str] = 'id',
                  allowed_methods=json_api_spec_http_methods.HTTP_ACTIONS,
                  permission_classes: Sequence[Type[BasePermission]] = None,
-                 authentication_classes: Sequence[Type[BaseAuthentication]] = None):
+                 authentication_classes: Sequence[Type[BaseAuthentication]] = None,
+                 raw_items=False):
         self._allowed_methods = [*allowed_methods]
         self._resource_name = action_name
+        self._raw_items = not raw_items
         self._unique_identifier = unique_identifier
         self._permission_classes = permission_classes or []
         self._authentication_classes = authentication_classes or []
@@ -487,7 +513,7 @@ class JsonApiResourceViewBuilder:
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_DELETE)
         return self
 
-    def on_list(self, list_callback: Callable[[Request], Tuple[List, int, int]] = None) -> 'JsonApiResourceViewBuilder':
+    def on_list(self, list_callback: Callable[[Request], Tuple[List, List, int, int]] = None) -> 'JsonApiResourceViewBuilder':
         self._on_list_callback = list_callback
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_GET)
         return self
@@ -511,20 +537,35 @@ class JsonApiResourceViewBuilder:
 
             if self._on_update_callback is not None:
                 data, status = self._on_update_callback(request, identifier, data, *args, **kwargs)
-                return Response(data={'id': identifier, 'type': self._resource_name, 'attributes': data}, status=status)
+                if self._raw_items:
+                    return Response(data={'id': identifier, 'type': self._resource_name, 'attributes': data},
+                                    status=status)
+                return Response(data=data, status=status)
 
         def create(view, request, *args, **kwargs):
-            data = json.loads(request.body).get('data', {})
+            data = json.loads(request.body).get('data', {}) \
+                if 'multipart' not in request.content_type else request.body
             if self._on_create_callback is not None:
                 data, identifier, status = self._on_create_callback(request, data, *args, **kwargs)
-                return Response(
-                    data={'id': identifier, 'type': self._resource_name, 'attributes': data}, status=status)
+                if self._raw_items:
+                    return Response(data={'id': identifier, 'type': self._resource_name, 'attributes': data},
+                                    status=status)
+                return Response(data=data, status=status)
 
         def _list(view, request, *args, **kwargs):
             params = request.query_params
+            filters = []
+            for key, value in params.items():
+                match = FILTER_REGEX.match(key)
+                if match:
+                    filters.append({'field': match.groupdict()['field'],
+                                    'op': FILTER_MAP.get(match.groupdict()['op'], '=='),
+                                    'value': ast.literal_eval(value)})
             page = params.get('page_number', 1)
+            include = params.get('include', '')
+            includes = include.split(',') if include else []
             if self._on_list_callback is not None:
-                data, count, status = self._on_list_callback(request, page, *args, **kwargs)
+                data, included, count, status = self._on_list_callback(request, page, filters, includes, *args, **kwargs)
                 pages = math.ceil(count / 50)
                 return Response(data={'links': {
                     "first": f"/api/{self._resource_name}?page_number=1",
@@ -533,8 +574,10 @@ class JsonApiResourceViewBuilder:
                     "previous": None if page <= 1 else f"/api/{self._resource_name}?page_number={page - 1}",
                 }, 'data': [
                     {'id': item.get(self._unique_identifier, None), 'type': self._resource_name,
-                     'attributes': item} for
-                    item in data], 'meta': {
+                     'attributes': item} if self._raw_items else item for
+                    item in data],
+                    'included': included if included else [],
+                    'meta': {
                     'pagination': {
                         "page": page,
                         "pages": pages,
@@ -548,7 +591,10 @@ class JsonApiResourceViewBuilder:
 
             if self._on_get_callback is not None:
                 data, status = self._on_get_callback(request, identifier, *args, **kwargs)
-                return Response(data={'id': identifier, 'type': self._resource_name, 'attributes': data}, status=status)
+                if self._raw_items:
+                    return Response(data={'id': identifier, 'type': self._resource_name, 'attributes': data},
+                                    status=status)
+                return Response(data=data, status=status)
 
         patch_view_set = type(f'{self._resource_name}ChangeJSONApiActionViewSet', (ViewSet,), {
             'renderer_classes': (JSONRenderer, BrowsableAPIRenderer),
@@ -561,9 +607,9 @@ class JsonApiResourceViewBuilder:
             'http_method_names': list(map(lambda method: method.lower(), self._allowed_methods)) + ['head', 'options'],
             'permission_classes': self._permission_classes,
             'authentication_classes': self._authentication_classes,
-            'update': update,
-            'destroy': destroy,
-            'get': get
+            'update': update if self._on_update_callback else None,
+            'destroy': destroy if self._on_delete_callback else None,
+            'get': get if self._on_get_callback else None
         })
 
         get_view_set = type(f'{self._resource_name}RetrieveJSONApiActionViewSet', (ViewSet,), {
@@ -577,8 +623,8 @@ class JsonApiResourceViewBuilder:
             'http_method_names': list(map(lambda method: method.lower(), self._allowed_methods)) + ['head', 'options'],
             'permission_classes': self._permission_classes,
             'authentication_classes': self._authentication_classes,
-            'list': _list,
-            'create': create,
+            'list': _list if self._on_list_callback else None,
+            'create': create if self._on_create_callback else None,
         })
 
         urls = []
@@ -610,7 +656,7 @@ def json_api_view(resource_name: str,
                   authentication_classes: Optional[Sequence[Type[BaseAuthentication]]] = None,
                   urls_prefix: str = '',
                   urls_suffix: str = '',
-                  multiple_resource = True) -> FunctionType:
+                  multiple_resource=True) -> FunctionType:
     def decorator(func: Callable[[Request], Tuple[Dict, int]]):
         builder = JsonApiResourceViewBuilder(action_name=resource_name,
                                              allowed_methods=[method],
